use crate::thir::*;

use rustc_hir as hir;

crate trait ToRef {
    type Output;
    fn to_ref(self) -> Self::Output;
}

impl<'tcx> ToRef for &'tcx hir::Expr<'tcx> {
    type Output = ExprRef<'tcx>;

    fn to_ref(self) -> ExprRef<'tcx> {
        ExprRef::Thir(self)
    }
}

impl<'tcx> ToRef for &'tcx &'tcx hir::Expr<'tcx> {
    type Output = ExprRef<'tcx>;

    fn to_ref(self) -> ExprRef<'tcx> {
        ExprRef::Thir(&**self)
    }
}

impl<'tcx> ToRef for Expr<'tcx> {
    type Output = ExprRef<'tcx>;

    fn to_ref(self) -> ExprRef<'tcx> {
        ExprRef::Mirror(Box::new(self))
    }
}

impl<'tcx, T, U> ToRef for &'tcx Option<T>
where
    &'tcx T: ToRef<Output = U>,
{
    type Output = Option<U>;

    fn to_ref(self) -> Option<U> {
        self.as_ref().map(|expr| expr.to_ref())
    }
}

impl<'tcx, T, U> ToRef for &'tcx Vec<T>
where
    &'tcx T: ToRef<Output = U>,
{
    type Output = Vec<U>;

    fn to_ref(self) -> Vec<U> {
        self.iter().map(|expr| expr.to_ref()).collect()
    }
}

impl<'tcx, T, U> ToRef for &'tcx [T]
where
    &'tcx T: ToRef<Output = U>,
{
    type Output = Vec<U>;

    fn to_ref(self) -> Vec<U> {
        self.iter().map(|expr| expr.to_ref()).collect()
    }
}
